/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */


#include "denemo_objects.h"
typedef void (*GActionCallback) (GtkAction *action, gpointer data);
#define G_ACTIONCALLBACK(f) ((GActionCallback)(f)) 
 /* and the following typedefs are basically here for so that it's
 * possible to understand what my code is doing -- just as much for
 * my sake as yours!
 *
 * What can I say; I've done a lot of programming in Java and
 * SML/NJ; 
 * I like my type names to convey information. */

/* The ->data part of each objnode presently points to a DenemoObject */

typedef GList objnode;


typedef enum 
  {
    UNDO,
    REDO
  }unre_mode;

/* The idea here is to make everything recursive.  The dominant
   paradigm is a linked list.  Handy that there's such a nice
   precooked implementation of them in glib, eh?  Note, however, that
   the whole score isn't treated as a linked list of notes and other
   objects as it is in, say Rosegarden; instead, the program has a
   linked list of musical objects for each measure, and only then are
   measures linked into staffs.  That's my main beef with Rosegarden
   -- I don't tend to compose my stuff in the order in which it will
   eventually be played. As such, I like being able to start entering
   music in the middle of my score, then the beginning, then the end,
   or whatever, as appropriate.  */

typedef enum DenemoObjType
{
  CHORD,
  TUPOPEN,
  TUPCLOSE,
  CLEF,
  TIMESIG,
  KEYSIG,
  BARLINE,
  STEMDIRECTIVE,
  MEASUREBREAK,
  STAFFBREAK,
  DYNAMIC,
  GRACE_START,
  GRACE_END,
  LYRIC,
  FIGURE,
  FAKECHORD
}DenemoObjType;

/**
 * Enumeration for Tuplets type
 * 
 */
typedef enum tuplet_type{
	DUPLET,
	TRIPLET,
	QUADTUPLET,
	QUINTUPLET,
	SEXTUPLET,
	SEPTUPLET	
}tuplet_type;

/**
 * Enumeration for Denemo's input mode
 */
typedef enum input_mode {
  INPUTNORMAL,
  INPUTREST,
  INPUTBLANK,
  REPLACE,
  TRAVERSE
}input_mode;

/**
 * Denemo Action type currently used for undo/redo 
 * 
 */
typedef enum  action_type {
  ACTION_INSERT,
  ACTION_DELETE,
  ACTION_CHANGE
}action_type;

/**
 * Contains all the top-level information of an musical object
 * the object pointer contains the actual object
 */
typedef struct 
{
  DenemoObjType type;
  gchar *user_string;/* holds user's original text parsed to generated this 
			object */
  gint basic_durinticks;
  gint durinticks;
  gint starttick;
  gint starttickofnextnote;
  /* Allots extra space for accidentals or reverse-aligned notes if
   * the stem is down */
  gint space_before;
  gint minpixelsalloted;
  /* Holds x co-ordinate relative to the beginning of the measure. */
  gint x;
  gboolean isstart_beamgroup;
  gboolean isend_beamgroup;
  /* mudelaobjunion u; has become a pointer*/
  gpointer object;
  gboolean isinvisible;
}DenemoObject;


/**
 * Enumeration for a staffs context. 
 * Currently only four to chose from,  may change if Lilypond 
 * adds any more
 */
typedef enum 
{
  DENEMO_NONE=1,
  DENEMO_PIANO,
  DENEMO_GROUP,
  DENEMO_CHOIR
}DenemoContext;


/**
 * The ->data part of each measurenode points to an objlist, which is
 * a list of the musical objects in that measure. 
 */
typedef GList measurenode;


/**
 * DenemoStaff contains all the information relating to a musical staff
 * 
 */
typedef struct 
{
  measurenode *measures;
  gint sclef;
  gint skey;
  gint skey_isminor;  //25112005 Changed use to incorporate modes values 0 - major, 1 - minor, 2 - mode
  gint skeyaccs[7];
  gint stime1;
  gint stime2;
  gint leftmost_clefcontext;
  gint leftmost_keysigcontext;
  gint leftmost_keywidth;
  gint leftmost_time1context;
  gint leftmost_time2context;
  gint leftmost_keyaccs[7];
  gint leftmost_stem_directive;
  /*New variable to define the context in which this staff appears*/
  DenemoContext context; 
  /*
   * Staff Parameters
   * Added Adam Tee 27/1/2000, 2001 
   */
  gint no_of_lines;
  gint transposition;
  gint pos_in_half_lines;
  gint volume;
  /* Back to Hiller stuff */
  GString *staff_name;
  /* RTS: I've introduced the staff name here, the other two are versions
     of the voice name; however I'm still sticking to the unwritten convention
     that each staff's voices are contiguous in si->thescore. Without this you
     can't have same named voices in different staffs. */
  GString *denemo_name;
  GString *lily_name;
  GString *midi_instrument;

  gint space_above;
  gint space_below;
  gint voicenumber;
  /* presently set to 2 for any non-primary voices; we might want to
   * change that, though */
  measurenode ** is_parasite; /* points to address of host staff's measures 
				 field if measures are owned by another 
				 staff */

  gint nummeasures; /*Number of measures in the staff*/
  
}DenemoStaff;

/* The ->data part of each staffnode points to a staff structure */

typedef GList staffnode;
typedef staffnode *score;

/* a pair of staffs, used to relate two staffs together */
typedef struct staff_info
{
  DenemoStaff *main_staff; /* eg the bass line or the vocal part */
  DenemoStaff *related_staff; /* eg the figures for the bass or the lyrics*/
}
staff_info;


/**
 * Union for the two types of keypress related functions
 */
union func_union
{
  GtkFunction nocallback;  /*Not associated with a menued item */
  GActionCallback callback; /*Associated with a menued item */
};

/**
 * This structure is what's stored in the keymap data structure, both
 * in the array and in the hash tables.  Note that some redundant
 * information is stored in either case, but either one structure or
 * the other will need or find it useful to have each piece of information. 
 */
typedef struct KeybindingInfo
{
  gint keyval;
  gint state;
  gint command_number;
  gint callback_action;
  union func_union func;
}KeybindingInfo;

typedef struct DenemoKeymap
{
  GList **commands;
  GHashTable *quick_lookup_hashes;
}keymap;


#define MAX_HISTORY 10
/**
 * DenemoPrefs holds information on user preferences. 
 */
typedef struct DenemoPrefs
{
  GString *lilypath;
  GString *midiplayer;
  GString *audioplayer;
  gboolean playbackoutput;
  gboolean immediateplayback;
  gboolean lilyentrystyle;
  gboolean createclones;
  gboolean articulation_palette;
  gboolean notation_palette;
  gboolean saveparts; /*Automatically save parts*/
  /*Auto save data*/
  gboolean autosave;
  gint autosave_timeout;
  GString *browser;
  GString *csoundcommand;
  GString *csoundorcfile;
  gboolean rtcs;
  GString *pdfviewer;
  GString *texteditor;
  GString *denemopath;
  GQueue *history; //Queue to contain recently opened files
  keymap *the_keymap;
  GString *lilyversion;
}DenemoPrefs;

/**
 * Contains all the lilypond header information
 *
 */
typedef struct LilypondHeaderInfo
{
/* Mudela file header information */
  GString *title;
  GString *subtitle;
  GString *poet;
  GString *composer;
  GString *meter;
  GString *opus;
  GString *arranger;
  GString *instrument;
  GString *dedication;
  GString *piece;
  GString *head;
  GString *copyright;
  GString *footer;
  GString *tagline;
/*  GString *; */

}LilypondHeaderInfo;

typedef enum 
{
  WOODWIND,
  BRASS,
  STRINGS,
  VOCALS,
  PITCHEDPERCUSSION,
  PLUCKEDSTRINGS,
  KEYBOARDS,
  NONE
}InstrumentType;

/**
 * Structure to contain the list of Instruments for the score
 * configuration wizard
 *
 */
typedef struct 
{
	GString *name;
	GString *midiinstrument;
	gint sclef;
	gint transposition;
	gint numstaffs;

}InstrumentConfig;

/**
 * Stores global instrument type and a list of InstrumentConfig structures
 */
typedef struct 
{
	InstrumentType type;
	GList *instruments;  // List to contain a list of Instruments of given type
}InstrumentList;

/**
 * Contains data required for undo/redo operation 
 * Borrowed idea from GScore
 */
typedef struct unre_data
{
  gpointer object;    /* pointer to object to be undone/redone */
  gint staffnum;      /* staff number */     
  gint measurenum;    /* measure number */
  gint position;      /* position in bar */
  enum action_type action; /*action type */

}unre_data;
 

/**
 * Structure to hold bookmark information
 * Id - gint
 * Bar - gint
 * Staff - gint
 */
typedef struct Bookmark
{
  gint id;
  gint bar;
  gint staff;
}Bookmark;

/** 
 * Contains paper score configuration
 *
 */
typedef struct DenemoScoreConfig
{
  GString *papersize;
  gint fontsize;
  GString	*lilyversion;
  gboolean orientation;
	
}DenemoScoreConfig;


/*
 *It contains
 * within it a DenemoPrefs *, and when Denemo supports editing
 * multiple scores simultaneously, each score's scoreinfo structure will
 * contain a pointer to the same single DenemoPrefs structure.
 *
 * I originally hit on this idea as a hack -- everything in Denemo was
 * coded to pass around struct scoreinfo *s, and I didn't want to
 * change this around -- but then I realized that it was a reasonably
 * good idea.  Basically all operations need access to the scoreinfo
 * structure, and a handful need access to the DenemoPrefs structure as
 * well. Only the preferences dialog needs access to _just_ the
 * DenemoPrefs structure.  So, why not keep things simple and always pass
 * around scoreinfos? */
 
/**
 * DenemoScore contains the information relating to a musical score
 */
typedef struct DenemoScore
{
  /*Indicator for open file readonly*/
  gint readonly;
  GList *curlilynode; /* the node of the lily parse tree on display 
			 in textwindow */
  GList *lily_file; /* root of lily file parse, see lilyparser.y etc  */
  DenemoScoreConfig *config; /*contains the paper configuration of the score*/
 
  gint leftmeasurenum; /* start at 1 */
  gint rightmeasurenum;/* start at 1 */
  gint top_staff;
  gint bottom_staff;
  gint measurewidth;
  /* List of all minimum measure widths */
  GList *measurewidths;
  gint widthtoworkwith;
  gint staffspace;

  /* Fields that have more to do with the data model and its manipulation,
   * though they may be modified by side-effects of the drawing routines */
  score thescore;
  
  staffnode *currentprimarystaff;
  staffnode *currentstaff;
  gint currentstaffnum;
  measurenode *currentmeasure;
  gint currentmeasurenum;/* start at 1 */
  objnode *currentobject;
  /* currentobject points to the note preceding the cursor when the
   * cursor is appending. == NULL only when currentmeasure is empty. */
  gint cursor_x;
  gint cursor_y;
  gint staffletter_y;
  gint maxkeywidth;
  gboolean cursor_appending;
  
  gboolean cursoroffend;
  gint cursorclef;
  gint cursoraccs[7];
  gint cursortime1;
  gint cursortime2;
  gint curmeasureclef;
  gint curmeasurekey;
  gint curmeasureaccs[7];
  gint nextmeasureaccs[7];
  /* These are used for determining what accidentals should be there
   * if the cursor advances to the next measure from the next "insert chord"
   * operation */
  gint curmeasure_stem_directive;

  gboolean is_grace_mode;
  /* Is there a figured bass present, is so this relates the bass
   * with its figures staff, if one is present */
  staff_info * has_figures;
  staff_info *has_fakechords;
  /* Now stuff that's used for marking areas */
  gint markstaffnum;
  gint markmeasurenum;
  gint markcursor_x;
  gint firststaffmarked;
  gint laststaffmarked;
  gint firstmeasuremarked;
  gint lastmeasuremarked;
  gint firstobjmarked;
  gint lastobjmarked;

  gboolean haschanged;

  /*Temp field for testing for a slur*/
  gboolean is_slured;

  /* Fields used for MIDI playback */
  gint tempo;
  gint start;
  gint end;
  gint stafftoplay;

  LilypondHeaderInfo *headerinfo;
  
  /* Filename */
  GString *filename;
  GString *autosavename;

 
  
  GList *savebuffer;
  /* Holds List of Plugin Names */
  

  /* a list of scoreinfos for each scoreblock in the (lilypond) file
     the last one is always the one currently at the address hardwired
     into the display functions.
     NULL if just one scoreblock in file */
  GList *scoreblocks;

  /*list of undo data */
  GQueue *undodata;
  GQueue *redodata;
  gint undo_redo_mode;

  
  DenemoScoreConfig *sconfig;
 
  GList *bookmarks;
  
  GList *Instruments;
  
}DenemoScore;

/**
 * DenemoGUI contains all the GUI information as 
 * well as a pointer to the DenemoScore structure.
 * 
 * It is done this way to allow abstraction from GUI specific 
 * and Score specific functions
 */
typedef struct DenemoGUI
{
		
  /* Fields used fairly directly for drawing */
  GtkWidget *window;
  GtkWidget *scorearea;
  GdkPixmap *pixmap;
  GtkObject *vadjustment;
  GtkWidget *vscrollbar;
  GtkObject *hadjustment;
  GtkWidget *hscrollbar;
  GtkWidget *menubar;
  GtkWidget *statusbar;
  gint status_context_id;
  /* text of lily input string corresponding to curobj */
  GtkTextBuffer *textbuffer; 
  GtkWidget *textwindow; /* top level window containing lily textwindow */
  GtkWidget *textview; /* text entry containing lily text to be edited */
  GtkWidget *musicdatabutton; /* button for editing music data textually */
  gulong sigid; /* signal handler id for when text is changed */

  GtkWidget *musictext; /* status bar with text of lily input string 
			   corresponding to music item at cursor */
	
  /* UI manager */
  GtkUIManager *ui_manager;
  
  GList *plugins;
  gint plugincounter;
  
   /* And the DenemoPrefs pointer */
  DenemoPrefs *prefs;
	 /* Articulation palette window */
  GtkWidget* articulation_palette;
  input_mode mode; /*Input mode for Score */
  GtkWidget *progressbar;  	  
  DenemoScore *si;
}DenemoGUI;
